#include "cameraeventhandler.h"
#include "mycamera.h"

const Ogre::Real CameraEventHandler::turboModifier(10);
const QPoint     CameraEventHandler::invalidMousePoint(-1,-1);

CameraEventHandler::CameraEventHandler(MyCamera *targetCamera)
:camera(targetCamera)
{
}

bool CameraEventHandler::keyPressEvent(QKeyEvent *e)
{
	static QMap<int, Ogre::Vector3> keyCoordModificationMapping;
	static bool mappingInitialised = false;
	
	if(!mappingInitialised)
	{
		keyCoordModificationMapping[Qt::Key_Z] 		  = Ogre::Vector3( 0, 0,-5);
		keyCoordModificationMapping[Qt::Key_S] 		  = Ogre::Vector3( 0, 0, 5);
		keyCoordModificationMapping[Qt::Key_Q] 		  = Ogre::Vector3(-5, 0, 0);
		keyCoordModificationMapping[Qt::Key_D] 		  = Ogre::Vector3( 5, 0, 0);
		keyCoordModificationMapping[Qt::Key_PageUp]   = Ogre::Vector3( 0, 5, 0);
		keyCoordModificationMapping[Qt::Key_PageDown] = Ogre::Vector3( 0,-5, 0);
		
		mappingInitialised = true;
	}
	
	QMap<int, Ogre::Vector3>::iterator keyPressed =
		keyCoordModificationMapping.find(e->key());
	if(keyPressed != keyCoordModificationMapping.end())
	{
		const Ogre::Vector3 &actualCamPos = camera->getPosition();
		camera->setPosition(actualCamPos + keyPressed.value());
		
		return true;
	}
	
	return false;
}

bool CameraEventHandler::mouseMoveEvent(QMouseEvent *e)
{
    if(e->buttons().testFlag(Qt::LeftButton) && oldPos != invalidMousePoint)
    {
        const QPoint &pos = e->pos();
        Ogre::Real deltaX = pos.x() - oldPos.x();
        Ogre::Real deltaY = pos.y() - oldPos.y();
        
        if(e->modifiers().testFlag(Qt::ControlModifier))
        {
            deltaX *= turboModifier;
            deltaY *= turboModifier;
        }
        
        Ogre::Vector3 camTranslation(deltaX, deltaY, 0);
        const Ogre::Vector3 &actualCamPos = camera->getPosition();
        camera->setPosition(actualCamPos + camTranslation);
        
        oldPos = pos;
		return true;
    }
	
	return false;
}

bool CameraEventHandler::mousePressEvent(QMouseEvent *e)
{
    if(e->buttons().testFlag(Qt::LeftButton))
    {
        oldPos = e->pos();
		return true;
    }
	
	return false;
}

bool CameraEventHandler::mouseReleaseEvent(QMouseEvent *e)
{
    if(!e->buttons().testFlag(Qt::LeftButton))
    {
        oldPos = invalidMousePoint;
		return true;
    }
	
	return false;
}

bool CameraEventHandler::wheelEvent(QWheelEvent *e)
{
    Ogre::Vector3 zTranslation(0,0, -e->delta() / 60);
    
    if(e->modifiers().testFlag(Qt::ControlModifier))
    {
        zTranslation.z *= turboModifier;
    }

    const Ogre::Vector3 &actualCamPos = camera->getPosition();
    camera->setPosition(actualCamPos + zTranslation);
    
    return true;
}
