#include "mainwindow.h"

#include "coordinate3Dmodifier.h"
#include "ogrewidget.h"
#include "eventhandler.h"
#include "cameraeventhandler.h"
#include "mycamera.h"

MainWindow::MainWindow()
:ogreWidget(0)
{
	ogreWidget = new OgreWidget;
	coordModifier = new Coordinate3DModifier;
	
	createActionMenus();
	createDockWidget();
	
	setCentralWidget(ogreWidget);
}

void MainWindow::createActionMenus()
{
	QAction *changeColorAct = new QAction("Changer la couleur de fond", this);
	connect(changeColorAct, SIGNAL(triggered()), this, SLOT(chooseBgColor()));
	
	QAction *moveCamModeAct = new QAction("Deplacement de la camera", this);
	moveCamModeAct->setCheckable(true);
	moveCamModeAct->setChecked(false);
	connect(moveCamModeAct, SIGNAL(toggled(bool)), this, SLOT(moveCamModeToggled(bool)));
	
	QAction *closeAct = new QAction("Quitter", this);
	connect(closeAct, SIGNAL(triggered()), this, SLOT(close()));
	
	QMenu *menu = menuBar()->addMenu("Divers");
	menu->addAction(changeColorAct);
	menu->addAction(moveCamModeAct);
	menu->addAction(closeAct);
}

void MainWindow::createDockWidget()
{
	coordModifierDock = new QDockWidget("Camera Position", this);
	coordModifierDock->setAllowedAreas(Qt::TopDockWidgetArea |
								 Qt::BottomDockWidgetArea);
	coordModifierDock->setFeatures(QDockWidget::DockWidgetMovable |
							QDockWidget::DockWidgetFloatable);
	coordModifierDock->setWidget(coordModifier);
	
	coordModifierDock->setVisible(false);
	
	addDockWidget(Qt::TopDockWidgetArea, coordModifierDock);
}

void MainWindow::chooseBgColor()
{
	QColor c = QColorDialog::getColor();
	ogreWidget->setBackgroundColor(c);
}

void MainWindow::moveCamModeToggled(bool on)
{
	if(on)
	{
		MyCamera *cam = ogreWidget->getCamera();
		CameraEventHandler *camEventHandler = new CameraEventHandler(cam);
		ogreWidget->setEventHandler(camEventHandler);
		
		coordModifier->setNewCoordinate(cam->getPosition());
		coordModifierDock->setVisible(true);
	
		connect(coordModifier, SIGNAL(coordinateChanged(const Ogre::Vector3&)),
				cam, SLOT(setPosition(const Ogre::Vector3&)));
		connect(cam, SIGNAL(positionChanged(const Ogre::Vector3&)),
				coordModifier, SLOT(setNewCoordinate(const Ogre::Vector3&)));
	}
	else
	{
		ogreWidget->setEventHandler(0);
		
		coordModifierDock->setVisible(false);
		coordModifierDock->disconnect();
	}
}
